import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../../generated/colors.gen.dart';

class KntSelectionPopupPresenter extends StatefulWidget {
  const KntSelectionPopupPresenter({
    Key? key,
    required this.options,
    required this.child,
    this.style,
    this.duration,
    this.onPressed,
    this.openWithTap = false,
  }) : super(key: key);

  final List<KntSelectionPopupOption> options;
  final Widget child;
  final KntSelectionPopupStyle? style;

  /// Duration that needs to show popup
  final Duration? duration;

  /// Callback at the time before showing popup
  final Function? onPressed;

  /// Open with tap instead of long press.
  final bool openWithTap;

  @override
  _KntSelectionPopupPresenterState createState() =>
      _KntSelectionPopupPresenterState();
}

class _KntSelectionPopupPresenterState
    extends State<KntSelectionPopupPresenter> {
  final GlobalKey _containerKey = GlobalKey();
  Offset _childOffset = Offset.zero;
  Size? _childSize;

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      key: _containerKey,
      onTap: () async {
        widget.onPressed?.call();
        if (widget.openWithTap) {
          await openMenu(context);
        }
      },
      onLongPress: () async {
        if (!widget.openWithTap) {
          await openMenu(context);
        }
      },
      child: widget.child,
    );
  }

  Future openMenu(BuildContext context) async {
    calculateOffset();
    await Navigator.push(
      context,
      PageRouteBuilder(
        transitionDuration:
            widget.duration ?? const Duration(milliseconds: 100),
        pageBuilder: (context, animation, secondaryAnimation) {
          animation = Tween(begin: 0.0, end: 1.0).animate(animation);
          return FadeTransition(
            opacity: animation,
            child: KntSelectionPopup(
              options: widget.options,
              child: widget.child,
              childOffset: _childOffset,
              childSize: _childSize ?? Size.zero,
              style: widget.style,
            ),
          );
        },
        fullscreenDialog: true,
        opaque: false,
      ),
    );
  }

  void calculateOffset() {
    RenderBox renderBox =
        _containerKey.currentContext!.findRenderObject() as RenderBox;
    Size size = renderBox.size;
    Offset offset = renderBox.localToGlobal(Offset.zero);
    setState(() {
      _childOffset = Offset(offset.dx, offset.dy);
      _childSize = size;
    });
  }
}

class KntSelectionPopup extends StatelessWidget {
  const KntSelectionPopup({
    Key? key,
    required this.options,
    required this.child,
    required this.childOffset,
    required this.childSize,
    this.style,
  }) : super(key: key);

  final List<KntSelectionPopupOption> options;
  final Widget child;
  final Offset childOffset;
  final Size childSize;
  final KntSelectionPopupStyle? style;

  @override
  Widget build(BuildContext context) {
    // From style
    final menuOffset = style?.menuOffset;
    final menuWidth = style?.menuWidth;
    final menuOptionExtent = style?.menuOptionExtent ?? 50.w;
    final menuOptionPadding = style?.menuOptionPadding ??
        EdgeInsets.symmetric(
          horizontal: 14.w,
          vertical: 8.w,
        );
    final bottomOffsetHeight = style?.bottomOffsetHeight;
    final menuBoxDecoration = style?.menuDecoration;
    final menuBorderRadius =
        style?.menuBorderRadius ?? BorderRadius.circular(5.w);
    final blurSize = style?.blurSize;
    final blurBackgroundColor = style?.blurBackgroundColor;
    final animateMenu = style?.needAnimate ?? true;
    final animateDuration =
        style?.animateDuration ?? const Duration(milliseconds: 200);
    // Calculating
    final maxMenuHeight = 0.4.sh;
    final maxMenuWidth = menuWidth ?? 0.7.sw;
    final optionsHeight = options.length * menuOptionExtent;
    final menuHeight =
        optionsHeight < maxMenuHeight ? optionsHeight : maxMenuHeight;
    var leftOffset = (childOffset.dx + maxMenuWidth) < 1.sw
        ? childOffset.dx
        : (childOffset.dx - maxMenuWidth + childSize.width);
    if (leftOffset < 0) {
      leftOffset = menuOffset ?? 0;
    }
    final topOffset = (childOffset.dy + menuHeight + childSize.height) <
            1.sh - bottomOffsetHeight!
        ? childOffset.dy + childSize.height + menuOffset!
        : childOffset.dy - menuHeight - menuOffset!;
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: Container(
        child: Stack(
          fit: StackFit.expand,
          children: <Widget>[
            GestureDetector(
              onTap: () {
                Navigator.pop(context);
              },
              child: BackdropFilter(
                filter: ImageFilter.blur(
                  sigmaX: blurSize ?? 4,
                  sigmaY: blurSize ?? 4,
                ),
                child: Container(
                  color:
                      (blurBackgroundColor ?? ColorName.black).withOpacity(0.7),
                ),
              ),
            ),
            Positioned(
              left: leftOffset,
              top: topOffset,
              child: TweenAnimationBuilder<double>(
                duration: animateDuration,
                builder: (BuildContext context, double? value, Widget? child) {
                  return Transform.scale(
                    scale: value,
                    child: child,
                  );
                },
                tween: Tween(begin: 0.0, end: 1.0),
                child: Container(
                  width: maxMenuWidth,
                  height: menuHeight,
                  decoration: menuBoxDecoration ??
                      BoxDecoration(
                        color: Colors.grey.shade200,
                        borderRadius: menuBorderRadius,
                        boxShadow: [
                          const BoxShadow(
                            color: Colors.black38,
                            blurRadius: 10,
                            spreadRadius: 1,
                          )
                        ],
                      ),
                  child: ClipRRect(
                    borderRadius: menuBorderRadius,
                    child: ListView.builder(
                      itemCount: options.length,
                      padding: EdgeInsets.zero,
                      physics: const BouncingScrollPhysics(),
                      itemBuilder: (context, index) {
                        KntSelectionPopupOption item = options[index];
                        Widget listItem = GestureDetector(
                          onTap: () {
                            Navigator.pop(context);
                            item.onPressed();
                          },
                          child: Container(
                            alignment: Alignment.center,
                            margin: const EdgeInsets.only(bottom: 1),
                            color: item.backgroundColor ?? ColorName.white,
                            height: menuOptionExtent,
                            child: Padding(
                              padding: menuOptionPadding,
                              child: Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                children: <Widget>[
                                  item.title,
                                  if (item.trailingIcon != null) ...[
                                    item.trailingIcon!
                                  ]
                                ],
                              ),
                            ),
                          ),
                        );
                        if (animateMenu) {
                          return TweenAnimationBuilder<double>(
                            builder: (context, double value, child) {
                              return Transform(
                                transform: Matrix4.rotationX(1.5708 * value),
                                alignment: Alignment.bottomCenter,
                                child: child,
                              );
                            },
                            tween: Tween(begin: 1.0, end: 0.0),
                            duration: Duration(milliseconds: index * 200),
                            child: listItem,
                          );
                        } else {
                          return listItem;
                        }
                      },
                    ),
                  ),
                ),
              ),
            ),
            Positioned(
              top: childOffset.dy,
              left: childOffset.dx,
              child: AbsorbPointer(
                child: Container(
                  width: childSize.width,
                  height: childSize.height,
                  child: child,
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class KntSelectionPopupStyle {
  const KntSelectionPopupStyle({
    this.menuOffset,
    this.menuWidth,
    this.menuOptionExtent,
    this.menuOptionPadding,
    this.bottomOffsetHeight,
    this.menuDecoration,
    this.menuBorderRadius,
    this.blurSize,
    this.blurBackgroundColor,
    this.needAnimate,
    this.animateDuration,
  });

  final double? menuOffset;
  final double? menuWidth;
  final double? menuOptionExtent;
  final EdgeInsets? menuOptionPadding;
  final double? bottomOffsetHeight;
  final BoxDecoration? menuDecoration;
  final BorderRadius? menuBorderRadius;
  final double? blurSize;
  final Color? blurBackgroundColor;
  final bool? needAnimate;
  final Duration? animateDuration;
}

class KntSelectionPopupOption {
  const KntSelectionPopupOption({
    required this.title,
    this.backgroundColor,
    this.trailingIcon,
    required this.onPressed,
  });

  final Widget title;
  final Color? backgroundColor;
  final Icon? trailingIcon;
  final Function onPressed;
}
